//
//  UIImage+PaymentButton.m
//  Mobile Buy SDK
//
//  Created by Shopify.
//  Copyright (c) 2015 Shopify Inc. All rights reserved.
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the "Software"), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
//

#import "UIImage+PaymentButton.h"

@implementation UIImage (PaymentButton)

+ (UIImage *)paymentButtonImageWithFrame:(CGRect)originalFrame style:(PaymentButtonStyle)style type:(PaymentButtonType)type
{
	if (CGRectIsEmpty(originalFrame)) {
		return nil;
	}
	
	UIColor *foregroundColor = style == PaymentButtonStyleBlack ? [UIColor whiteColor] : [UIColor blackColor];
	CGRect frame = CGRectMake(0, 0, CGRectGetHeight(originalFrame) * 3.25f, CGRectGetHeight(originalFrame));
	
	UIGraphicsBeginImageContextWithOptions(CGSizeMake(CGRectGetWidth(frame), CGRectGetHeight(frame)), NO, [[UIScreen mainScreen] scale]);
	
	[foregroundColor setFill];
	
	switch (type) {
		case PaymentButtonTypeBuy:
		case PaymentButtonTypeSetup: {
			//// Subframes
			CGRect buyWithApplyPay = CGRectMake(CGRectGetMinX(frame) + (CGRectGetWidth(frame) * 0.09790f + 0.5f), CGRectGetMinY(frame) + (CGRectGetHeight(frame) * 0.25000f - 0.32f) + 0.82f, (CGRectGetWidth(frame) * 0.89055f + 0.15f) - (CGRectGetWidth(frame) * 0.09790f + 0.5f) + 0.35f, (CGRectGetHeight(frame) * 0.75414f + 0.5f) - (CGRectGetHeight(frame) * 0.25000f - 0.32f) - 0.82f);
			
			//// buyWithApplyPay
			{
				//// aPay Drawing
				UIBezierPath* aPayPath = UIBezierPath.bezierPath;
				[aPayPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.98657f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.26807f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.96306f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.58942f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95897f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.64744f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.96160f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.60888f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.96022f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.62872f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95722f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.67462f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95839f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.65699f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95781f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.66617f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95679f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.67462f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95496f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.64671f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95620f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.66581f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95562f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.65626f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95109f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.59162f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95372f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.62761f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95241f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.60925f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.92598f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.26807f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.91255f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.26807f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94846f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.73301f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94956f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.75138f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94941f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.74440f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94956f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.74954f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94846f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77011f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94956f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.75213f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94948f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.75579f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94087f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.84135f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94620f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.79802f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94365f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.82189f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.93299f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.88910f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.93795f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.86118f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.93532f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.87734f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.92452f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.92252f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.93029f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.90268f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.92744f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.91407f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.91656f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.94345f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.92167f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.93096f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.91897f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.93794f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.91525f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.94638f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.91963f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 1.00000f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.92102f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.99743f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.92751f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.98238f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.92218f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.99523f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.92430f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.99046f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.93832f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.94418f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.93080f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.97430f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.93445f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.96144f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94737f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.89277f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94160f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.92949f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.94467f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.91223f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95533f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.82593f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95000f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.87330f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95270f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.85090f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.96299f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.74146f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.95788f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.80095f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.96044f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77268f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.97095f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.63789f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.96547f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71024f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.96817f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.67535f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 1.00000f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.26807f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.98657f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.26807f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath closePath];
				[aPayPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88831f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.60962f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88736f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.63679f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88831f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.61623f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88795f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.62541f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88400f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.67058f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88663f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.64818f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88546f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.65956f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87853f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.69996f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88255f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.68160f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88065f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.69115f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87079f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72089f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87634f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70877f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87371f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71539f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.86072f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72860f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.86787f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72567f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.86444f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72860f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85408f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72273f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85846f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72860f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85619f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72640f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84853f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70583f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85196f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71905f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85013f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71354f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84459f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.67756f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84692f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.69849f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84561f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.68894f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84313f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.63532f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84356f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.66617f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84313f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.65148f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84736f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.56995f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84313f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.60815f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84451f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.58611f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85860f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.53139f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85035f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.55306f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85415f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.54020f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87379f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.51413f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.86320f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.52258f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.86831f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.51707f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88671f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.51082f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87824f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.51230f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88255f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.51082f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88831f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.51082f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88831f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.60962f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath closePath];
				[aPayPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.90109f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70841f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.90087f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.64928f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.90094f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.68894f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.90087f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.66911f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.90087f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.45647f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.89955f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.38706f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.90087f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.43370f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.90043f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.41057f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.89437f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.32280f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.89861f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.36356f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.89685f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.34189f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88371f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27542f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.89174f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.30333f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88817f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.28754f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.86641f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.25706f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87919f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.26330f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87342f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.25706f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85152f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.26697f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.86123f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.25706f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85627f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.26037f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.83765f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.30003f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84678f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27358f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84211f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.28497f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.83663f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.30370f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84086f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.35365f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84240f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.34850f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85335f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.32243f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84568f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.33749f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84933f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.32867f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.86539f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.31324f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85729f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.31655f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.86138f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.31324f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87780f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.32757f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87065f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.31324f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87481f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.31802f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88459f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.36245f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88087f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.33711f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88313f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.34887f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88758f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40763f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88612f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.37678f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88714f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.39183f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88831f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.45133f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88809f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.42415f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88831f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.43884f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88831f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.45684f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84539f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.50348f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.86963f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.45646f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85510f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.47226f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.83006f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.64267f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.83524f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.53653f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.83006f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.58317f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.83189f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.69408f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.83006f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.65993f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.83072f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.67719f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.83736f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.73999f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.83313f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71134f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.83495f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72677f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84670f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77231f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.83984f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.75321f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.84298f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.76423f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85984f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.78479f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85050f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.78075f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.85488f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.78479f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87072f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77708f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.86371f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.78479f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.86736f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.78223f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87963f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.75762f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87401f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77231f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.87693f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.76570f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88642f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.73044f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88218f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.74954f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88452f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.74036f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88882f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71722f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88729f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72604f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88809f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72164f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.88934f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71722f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.89050f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77341f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.90262f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77341f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.90226f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.76460f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.90109f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70841f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.90160f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.74624f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.90123f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72751f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath closePath];
				[aPayPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.80364f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.38927f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.77641f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.43041f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.79721f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.41682f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.78802f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.43041f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76750f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.42857f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.77327f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.43041f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.77020f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.42965f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76115f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.42306f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76517f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.42747f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76305f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.42564f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76115f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12816f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76728f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12375f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76283f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12669f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76488f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12523f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.77831f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12155f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.77035f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12228f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.77400f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12155f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.79261f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.13074f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.78349f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12155f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.78831f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12448f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.80371f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.15791f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.79692f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.13698f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.80064f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.14616f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.81072f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.20345f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.80670f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.16966f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.80911f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.18509f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.81327f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.26992f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.81240f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.22218f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.81327f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.24458f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.80364f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.38927f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.81327f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.32244f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.80999f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.36246f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath closePath];
				[aPayPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.81524f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12449f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.80021f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.08042f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.81130f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.10540f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.80619f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.09071f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.77867f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.06500f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.79415f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.07014f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.78692f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.06500f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76290f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.06941f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.77298f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.06500f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76765f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.06646f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.74984f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.07859f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.75816f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.07197f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.75378f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.07528f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.74852f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.07969f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.74852f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77343f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76115f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77343f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76115f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48110f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.77553f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48661f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.76539f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48477f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.77020f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48661f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.79546f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.47302f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.78269f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48661f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.78940f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48220f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.81130f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.43152f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.80152f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46421f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.80685f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.44988f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.82196f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.36174f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.81576f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.41279f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.81941f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.38929f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.82583f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.26589f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.82452f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.33456f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.82583f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.30188f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.82306f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.18546f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.82583f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.23577f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.82488f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.20860f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.81524f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12449f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.82123f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.16196f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.81860f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.14175f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath closePath];
				[aPayPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.66713f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12450f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.67523f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.00000f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.67261f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.09144f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.67618f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.04591f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.65246f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.05913f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.66742f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.00183f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.65801f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.02644f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.64421f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.17958f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.64742f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.08814f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.64304f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.13478f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.66713f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.12450f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.65290f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.18326f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.66173f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.15755f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath closePath];
				[aPayPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.68961f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.41389f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.70633f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.26552f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.68940f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.31547f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.70560f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.26809f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.67801f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.18840f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.69721f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.19868f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.68304f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.18950f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.64837f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.22439f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.66596f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.18252f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.65450f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.22439f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.62282f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.19060f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.64224f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.22439f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.63282f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.18950f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.59078f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.28792f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.60968f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.19135f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.59756f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.22880f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.60063f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.68088f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.57712f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40765f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.58734f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.58430f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.62508f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77931f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.60713f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72789f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.61486f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.78114f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.65041f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.74735f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.63487f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77747f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.63859f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.74735f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.67596f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77820f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.66231f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.74735f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.66560f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77931f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.69969f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.68235f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.68655f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77710f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.69319f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.73009f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.71035f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.57181f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.70713f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.62763f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.71020f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.57438f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.68961f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.41389f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.71020f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.57106f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.68983f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.53215f * CGRectGetHeight(buyWithApplyPay))];
				[aPayPath closePath];
				[aPayPath fill];
				
				//// b Drawing
				UIBezierPath* bPath = UIBezierPath.bezierPath;
				[bPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04657f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71686f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06037f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.69262f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05249f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71686f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05708f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70878f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06533f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.62651f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06365f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.67682f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06533f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.65479f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06351f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.58575f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06533f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.61035f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06475f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.59640f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05869f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.56004f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06234f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.57510f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06073f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.56665f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05175f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.54682f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05672f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.55380f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05438f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.54939f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04365f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.54277f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04913f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.54425f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04643f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.54277f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.01328f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.54277f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.01328f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71685f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04657f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71686f * CGRectGetHeight(buyWithApplyPay))];
				[bPath closePath];
				[bPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.03986f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48659f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05621f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46639f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04745f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48659f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05285f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.47998f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06117f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40653f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05949f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.45317f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06117f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.43297f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05949f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.36393f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06117f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.38853f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06059f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.37458f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05490f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.34006f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05833f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.35364f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05679f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.34593f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04811f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.32941f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05292f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.33455f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05066f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.33087f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.03986f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.32684f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04548f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.32757f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04278f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.32684f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.01329f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.32684f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.01329f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48659f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.03986f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48659f * CGRectGetHeight(buyWithApplyPay))];
				[bPath closePath];
				[bPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.03628f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27065f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04314f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27102f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.03840f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27065f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04066f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27065f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05051f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27285f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04562f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27102f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04811f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27177f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05730f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27836f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05292f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27432f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05519f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27616f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06271f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.28901f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05942f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.28056f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06117f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.28424f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07103f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.32978f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06592f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.29856f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06869f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.31215f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07446f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.39368f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07329f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.34704f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07446f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.36834f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07059f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46309f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07446f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.42049f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07314f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.44363f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05964f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.50643f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06803f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48256f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06438f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.49688f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05964f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.50753f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07373f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.54976f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06577f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.51414f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07052f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.52846f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07862f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.62872f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07701f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.57143f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07862f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.59787f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07665f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.68234f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07862f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.64708f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07796f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.66471f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07088f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72824f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07533f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.69960f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.07344f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71502f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06139f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.76056f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06833f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.74183f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.06511f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.75248f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.04825f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77305f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05760f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.76901f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.05322f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77305f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.00000f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77305f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.00000f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27065f * CGRectGetHeight(buyWithApplyPay))];
				[bPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.03628f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27065f * CGRectGetHeight(buyWithApplyPay))];
				[bPath closePath];
				[bPath fill];
				
				//// u Drawing
				UIBezierPath* uPath = UIBezierPath.bezierPath;
				[uPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.14257f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.14257f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71539f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.14227f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71539f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.13264f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.76496f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.13979f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.73779f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.13658f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.75431f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.11877f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.78076f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.12870f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77562f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.12410f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.78076f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.10687f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77121f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.11402f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.78076f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.11008f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77745f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.09920f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.74403f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.10373f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.76497f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.10118f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.75578f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.09504f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70217f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.09723f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.73265f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.09585f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71832f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.09388f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.64855f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.09424f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.68601f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.09388f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.66801f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.09388f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.10577f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.10577f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.65552f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.10972f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70914f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.10577f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.67793f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.10709f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.69592f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.12045f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72861f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.11234f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72200f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.11592f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72861f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.12994f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72016f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.12410f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72861f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.12724f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72604f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.13658f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.69629f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.13256f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71466f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.13475f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70658f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.14052f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.65994f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.13833f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.68601f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.13965f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.67389f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.14184f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.61476f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.14140f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.64635f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.14184f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.63092f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.14184f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.15373f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.15373f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77305f * CGRectGetHeight(buyWithApplyPay))];
				[uPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.14257f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[uPath closePath];
				[uPath fill];
				
				//// y Drawing
				UIBezierPath* yPath = UIBezierPath.bezierPath;
				[yPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.19593f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.86596f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.19147f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.89571f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.19461f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.87808f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.19308f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.88799f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.18607f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.91260f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.18979f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.90378f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.18804f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.90929f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17906f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.91811f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.18410f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.91628f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.18176f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.91811f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17483f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.91664f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17768f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.91811f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17629f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.91736f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17082f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.91150f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17344f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.91554f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17213f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.91408f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17082f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.85678f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17425f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.86229f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17191f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.85861f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17308f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.86046f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17782f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.86522f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17541f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.86412f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17658f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.86522f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.18366f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.85678f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.18023f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.86522f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.18220f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.86229f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.18760f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.83217f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.18520f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.85127f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.18651f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.84282f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.19250f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77084f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.16381f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.17724f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.19841f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70694f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.19863f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70694f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.21892f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.23155f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.20009f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.82079f * CGRectGetHeight(buyWithApplyPay))];
				[yPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.19593f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.86596f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.19863f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.83878f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.19724f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.85384f * CGRectGetHeight(buyWithApplyPay))];
				[yPath closePath];
				[yPath fill];
				
				//// w Drawing
				UIBezierPath* wPath = UIBezierPath.bezierPath;
				[wPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.33951f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.32484f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48365f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.32455f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48365f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.31002f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.29710f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.27389f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.28703f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.30345f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70694f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.30374f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70694f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.31827f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.33126f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.34638f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70694f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.34667f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70694f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.36287f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.37521f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.35214f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[wPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.33951f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[wPath closePath];
				[wPath fill];
				
				//// i Drawing
				UIBezierPath* iPath = UIBezierPath.bezierPath;
				[iPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.39908f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[iPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.38725f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[iPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.38725f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[iPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.39908f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[iPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.39908f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[iPath closePath];
				[iPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.38725f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27065f * CGRectGetHeight(buyWithApplyPay))];
				[iPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.39908f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27065f * CGRectGetHeight(buyWithApplyPay))];
				[iPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.39908f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.34373f * CGRectGetHeight(buyWithApplyPay))];
				[iPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.38725f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.34373f * CGRectGetHeight(buyWithApplyPay))];
				[iPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.38725f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27065f * CGRectGetHeight(buyWithApplyPay))];
				[iPath closePath];
				[iPath fill];
				
				//// t Drawing
				UIBezierPath* tPath = UIBezierPath.bezierPath;
				[tPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.44850f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.44850f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46198f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43405f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46198f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43405f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.68784f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43441f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70474f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43405f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.69482f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43420f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70070f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43580f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71465f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43463f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.70915f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43514f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71245f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43850f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71906f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43638f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71685f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43733f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71833f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.44302f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72016f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43967f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71981f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.44113f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72016f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.44850f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72016f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.44850f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77305f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43937f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77305f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43149f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.76974f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43631f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77305f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43368f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77195f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.42616f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.75835f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.42930f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.76790f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.42755f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.76386f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.42317f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.73411f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.42485f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.75247f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.42383f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.74476f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.42222f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.69335f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.42251f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.72420f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.42222f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.71024f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.42222f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46198f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.40988f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46198f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.40988f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.42222f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.42222f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.30002f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43404f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.30002f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.43404f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[tPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.44850f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40910f * CGRectGetHeight(buyWithApplyPay))];
				[tPath closePath];
				[tPath fill];
				
				//// h Drawing
				UIBezierPath* hPath = UIBezierPath.bezierPath;
				[hPath moveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47354f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27065f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47354f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46272f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47383f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46272f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47770f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.43407f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47478f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.45133f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47602f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.44178f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.48339f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.41498f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47945f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.42636f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.48128f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.42012f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.48996f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40432f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.48551f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.41020f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.48770f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40653f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.49660f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40065f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.49222f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40176f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.49449f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40065f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.50850f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.41056f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.50135f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40065f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.50536f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.40396f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.51617f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.43811f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.51164f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.41717f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.51420f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.42636f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.52033f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.47998f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.51814f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.44986f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.51953f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46382f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.52150f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.53359f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.52113f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.49613f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.52150f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.51413f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.52150f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.50960f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.50960f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.52662f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.50566f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.47336f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.50960f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.50421f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.50828f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48622f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.49493f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.45353f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.50310f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46014f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.49952f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.45353f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.48551f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46198f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.49128f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.45353f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.48814f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.45647f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47887f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.48585f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.48281f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.46748f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.48062f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.47556f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47485f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.52220f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47712f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.49613f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47573f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.50825f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addCurveToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47354f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.56738f * CGRectGetHeight(buyWithApplyPay)) controlPoint1: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47397f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.53579f * CGRectGetHeight(buyWithApplyPay)) controlPoint2: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47354f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.55122f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47354f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.46164f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.77304f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.46164f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27064f * CGRectGetHeight(buyWithApplyPay))];
				[hPath addLineToPoint: CGPointMake(CGRectGetMinX(buyWithApplyPay) + 0.47354f * CGRectGetWidth(buyWithApplyPay), CGRectGetMinY(buyWithApplyPay) + 0.27065f * CGRectGetHeight(buyWithApplyPay))];
				[hPath closePath];
				[hPath fill];
			}
		}
			break;
		case PaymentButtonTypePlain: {
			//// applePay Drawing
			UIBezierPath* applePayPath = UIBezierPath.bezierPath;
			[applePayPath moveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.65968f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37344f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.63926f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.54881f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.63575f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.58038f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.63805f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.55940f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.63684f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.56998f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.63425f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59516f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.63523f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.58567f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.63471f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59060f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.63384f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59516f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.63229f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.57983f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.63338f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59024f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.63286f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.58513f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.62894f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.54991f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.63125f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.56962f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.63010f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.55958f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.60714f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37344f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.59549f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37344f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.62664f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62710f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.62762f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63714f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.62750f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63312f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.62762f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63586f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.62664f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.64717f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.62762f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63751f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.62756f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63951f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.62000f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.68604f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.62467f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.66232f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.62243f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.67546f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.61320f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.71196f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.61752f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.69681f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.61527f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.70557f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.60593f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.73039f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.61083f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.71962f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.60841f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.72564f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.59901f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.74188f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.60340f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.73513f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.60109f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.73896f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.59780f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.74353f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.60161f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.77273f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.60282f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.77127f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.60853f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.76306f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.60380f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.76999f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.60570f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.76744f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.61787f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.74207f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.61130f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.75868f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.61447f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.75156f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.62571f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.71397f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.62070f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.73422f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.62335f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.72473f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.63258f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.67765f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.62802f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.70357f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.63033f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.69134f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.63921f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63167f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.63477f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.66415f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.63702f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.64864f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.64613f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.57509f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.64140f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.61469f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.64376f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59553f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.67133f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37344f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.65968f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37344f * CGRectGetHeight(frame))];
			[applePayPath closePath];
			[applePayPath moveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57450f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.55976f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57364f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.57454f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.57450f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.56341f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.57421f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.56852f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57075f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59297f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.57300f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.58093f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.57202f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.58713f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.56597f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.60903f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.56949f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59900f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.56787f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.60429f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.55933f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62035f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.56412f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.61378f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.56187f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.61761f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.55057f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62454f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.55674f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62308f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.55380f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62454f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.54480f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62144f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.54855f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62454f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.54665f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62345f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.53996f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.61232f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.54296f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.61943f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.54134f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.61633f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.53656f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59680f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.53858f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.60830f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.53742f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.60301f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.53529f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.57363f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.53569f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59042f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.53529f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.58275f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.53898f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.53804f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.53529f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.55885f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.53656f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.54680f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.54872f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.51724f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.54152f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.52892f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.54480f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.52198f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.56187f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.50757f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.55270f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.51231f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.55714f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.50921f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57312f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.50592f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.56574f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.50647f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.56949f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.50592f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57450f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.50592f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.57358f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.50592f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.57404f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.50592f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57450f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.55976f * CGRectGetHeight(frame))];
			[applePayPath closePath];
			[applePayPath moveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.58557f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.61360f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.58540f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.58148f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.58546f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.60283f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.58540f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59206f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.58540f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.47618f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.58419f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.43841f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.58540f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.46377f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.58500f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.45100f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57969f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.40319f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.58344f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.42545f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.58189f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.41377f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57047f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37727f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.57744f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.39260f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.57439f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.38402f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.55547f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.36742f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.56660f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37070f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.56153f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.36742f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.54261f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37289f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.55103f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.36742f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.54671f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.36925f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.53056f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.39096f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.53846f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37654f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.53442f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.38256f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.52969f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.39297f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.53333f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.42016f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.53471f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.41742f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.54417f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.40319f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.53748f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.41140f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.54065f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.40647f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.55461f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.39808f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.54757f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.39972f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.55109f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.39808f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.56539f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.40574f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.55916f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.39808f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.56280f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.40063f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57127f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.42509f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.56799f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.41103f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.57000f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.41742f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57387f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.44954f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.57260f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.43275f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.57346f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.44096f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57450f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.47344f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.57427f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.45848f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.57450f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.46651f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57450f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.47637f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.53725f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.50191f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.55830f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.47618f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.54567f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.48476f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.52399f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.57783f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.52848f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.51980f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.52399f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.54535f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.52554f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.60575f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.52399f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.58714f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.52451f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59644f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.53033f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63093f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.52664f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.61524f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.52820f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62363f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.53840f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.64845f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.53246f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63805f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.53517f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.64407f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.54976f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.65520f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.54169f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.65302f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.54550f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.65520f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.55922f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.65119f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.55317f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.65520f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.55634f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.65375f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.56689f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.64042f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.56205f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.64845f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.56464f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.64480f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57289f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62564f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.56914f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63586f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.57116f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63093f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57496f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.61834f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.57364f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62327f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.57427f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62071f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57537f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.61834f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.57641f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.64900f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.58690f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.64900f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.58661f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.64407f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.58557f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.61360f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.58604f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63440f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.58569f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62400f * CGRectGetHeight(frame))];
			[applePayPath closePath];
			[applePayPath moveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.50109f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.43969f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.47750f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.46213f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.49550f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.45447f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.48754f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.46213f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.46972f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.46086f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.47474f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.46213f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.47208f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.46159f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.46418f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.45812f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.46770f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.46031f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.46586f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.45940f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.46418f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29716f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.46955f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29479f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.46568f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29625f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.46741f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29552f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.47906f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29333f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.47220f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29388f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.47537f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29333f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.49152f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29863f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.48362f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29333f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.48783f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29516f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.50115f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.31341f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.49521f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.30191f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.49844f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.30684f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.50726f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.33823f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.50374f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.31979f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.50582f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.32819f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.50939f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37436f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.50864f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.34845f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.50939f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.36067f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.50109f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.43969f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.50939f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.40301f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.50657f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.42490f * CGRectGetHeight(frame))];
			[applePayPath closePath];
			[applePayPath moveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.51112f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29516f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.49803f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.27107f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.50772f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.28476f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.50334f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.27654f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.47941f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.26268f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.49284f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.26560f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.48656f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.26268f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.46574f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.26505f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.47451f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.26268f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.46989f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.26359f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.45438f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.26998f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.46165f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.26651f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.45784f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.26815f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.45328f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.27070f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.45328f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.64900f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.46418f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.64900f * CGRectGetHeight(frame))];
			[applePayPath addLineToPoint: CGPointMake(CGRectGetMinX(frame) + 0.46418f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.48969f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.47675f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.49261f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.46787f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.49170f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.47208f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.49261f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.49394f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.48531f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.48292f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.49261f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.48869f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.49024f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.50772f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.46250f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.49924f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.48038f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.50386f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.47272f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.51695f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.42454f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.51159f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.45228f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.51470f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.43951f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.52029f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37217f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.51920f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.40958f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.52029f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.39206f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.51787f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.32837f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.52029f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.35574f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.51949f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.34096f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.51112f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29516f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.51631f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.31560f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.51401f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.30447f * CGRectGetHeight(frame))];
			[applePayPath closePath];
			[applePayPath moveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.38275f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29516f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.38973f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.22727f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.38743f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.27709f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.39054f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.25227f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.36995f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.25957f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.38293f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.22818f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.37474f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.24169f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.36286f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.32527f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.36557f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.27545f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.36182f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.30082f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.38275f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.29516f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.37036f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.32710f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.37803f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.31304f * CGRectGetHeight(frame))];
			[applePayPath closePath];
			[applePayPath moveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.40219f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.45301f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.41672f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37217f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.40202f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.39936f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.41603f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.37326f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.39215f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.33001f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.40876f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.33567f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.39654f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.33056f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.36643f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.34954f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.38172f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.32673f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.37174f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.34954f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.34429f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.33111f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.36113f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.34954f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.35294f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.33056f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.31649f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.38440f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.33287f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.33165f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.32238f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.35209f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.32503f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59845f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.30467f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.44954f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.31350f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.54572f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.34625f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.65229f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.33068f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62419f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.33737f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.65320f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.36822f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63477f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.35473f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.65120f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.35796f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63477f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.39037f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.65174f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.37849f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.63477f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.38143f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.65229f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.41095f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.59937f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.39954f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.65101f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.40530f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.62546f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.42018f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.53896f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.41741f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.56962f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.42007f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.54042f * CGRectGetHeight(frame))];
			[applePayPath addCurveToPoint: CGPointMake(CGRectGetMinX(frame) + 0.40219f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.45301f * CGRectGetHeight(frame)) controlPoint1: CGPointMake(CGRectGetMinX(frame) + 0.42001f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.53859f * CGRectGetHeight(frame)) controlPoint2: CGPointMake(CGRectGetMinX(frame) + 0.40236f * CGRectGetWidth(frame), CGRectGetMinY(frame) + 0.51743f * CGRectGetHeight(frame))];
			[applePayPath closePath];
			[applePayPath fill];
		}
			break;
	}
	
	UIImage *applePayImage = UIGraphicsGetImageFromCurrentImageContext();
	UIGraphicsEndImageContext();
	
	UIGraphicsBeginImageContextWithOptions(originalFrame.size, NO, [[UIScreen mainScreen] scale]);
	[applePayImage drawAtPoint:CGPointMake(CGRectGetMidX(originalFrame) - CGRectGetMidX(frame), 0)];
	UIImage *outputImage = UIGraphicsGetImageFromCurrentImageContext();
	UIGraphicsEndImageContext();
	
	outputImage = [outputImage imageWithRenderingMode:UIImageRenderingModeAlwaysOriginal];
	
	return outputImage;
}

@end
